/*
 * Licensed Materials - Property of HCL
 * UrbanCode Deploy
 * (c) Copyright HCL Technologies Ltd. 2018. All Rights Reserved.
 */
package com.urbancode.air.plugin.ispw.http

import groovy.json.JsonBuilder
import org.apache.http.HttpResponse
import org.apache.log4j.Logger

import com.urbancode.air.plugin.ispw.events.EventStatuses
import com.urbancode.air.plugin.ispw.models.SetInfo

/*
 * Client to interact with Compuware Enterprise Services ISPW REST API.
 */
abstract class CESClient {
    protected RestClientHelper restHelper
    protected String cesUrl
    protected String serverId
    protected Logger logger
    Properties props

    protected CESClient(Properties props) {
        this.props = props
        String cesUrl = "${props['cesHost'].trim()}:${props['cesPort'].trim()}"
        boolean trustAllCerts = Boolean.valueOf(props['trustAllCerts'])
        String authToken = props['authToken'].trim()
        String serverId = props['serverId'].trim()

        /* Configure CES server connection */
        restHelper = new RestClientHelper(cesUrl, trustAllCerts)
        restHelper.addRequestHeader("Authorization", authToken)
        this.serverId = serverId
        this.cesUrl = cesUrl
        logger = Logger.getLogger(getClass())
    }

    /* Get a SetInfo object from input properties
     * Values passed as null will not be serialized in the SetInfo object.
     */
    protected SetInfo getSetInfo() {
        /* All SetInfo properties are optional */
        String changeType = props['changeType']?.trim()
        String executionStatus = props['executionStatus']?.trim()
        String runtimeConfig = props['runtimeConfig']?.trim()
        String additionalProps = props['additionalProps']?.trim()

        SetInfo setInfo = new SetInfo()

        if (changeType) {
            setInfo.setChangeType(changeType)
        }

        if (executionStatus) {
            setInfo.setExecutionStatus(executionStatus)
        }

        if (runtimeConfig) {
            setInfo.setRuntimeConfiguration(runtimeConfig)
        }

        /* autoDeploy option is only valid for promotions */
        if (props.contains('autoDeploy')) {
            String autoDeploy = props['autoDeploy'] // ISPW REST API takes string values instead of boolean
            setInfo.setAutoDeploy(autoDeploy)
        }

        /* Callback configuration properties. These are optional. */
        String callbackUser = props['callbackUser']?.trim()
        String callbackPass = props['callbackPass']?.trim()
        String httpHeaders = props['httpHeaders']?.trim()
        String successCallbackUrl = props['successCallbackUrl']?.trim()
        String successCallbackMethod = props['successCallbackMethod']
        String successCallbackPayload = props['successCallbackPayload']?.trim()
        String failureCallbackUrl = props['failureCallbackUrl']?.trim()
        String failureCallbackMethod = props['failureCallbackMethod']
        String failureCallbackPayload = props['failureCallbackPayload']?.trim()

        if (callbackUser && callbackPass) {
            setInfo.setCredentials(callbackUser, callbackPass)
        }

        /* Set callback information */
        if (successCallbackUrl) {
            setInfo.generateEvents(
                EventStatuses.successStatuses,
                successCallbackUrl,
                successCallbackMethod,
                successCallbackPayload)
        }
        if (failureCallbackUrl) {
            setInfo.generateEvents(
                EventStatuses.failureStatuses,
                failureCallbackUrl,
                failureCallbackMethod,
                failureCallbackPayload)
        }

        if (additionalProps) {
            setInfo.parseAdditionalProps(additionalProps)
        }

        if (httpHeaders) {
            setInfo.parseHttpHeaders(httpHeaders)
        }

        return setInfo
    }
}
