/*
 * Licensed Materials - Property of HCL
 * UrbanCode Deploy
 * (c) Copyright HCL Technologies Ltd. 2018. All Rights Reserved.
 */
package com.urbancode.air.plugin.ispw.versions

import com.urbancode.air.plugin.ispw.http.CESClient
import com.urbancode.air.plugin.ispw.models.SetInfo
import org.apache.http.HttpResponse

/*
 * Client to hande operations on ISPW Sets.
 * Sets are created when operations against Assignments and Releases execute.
 * Using the setId you can perform actions (hold, release, unlock, or delete) on the Set.
 */
class SetClient extends CESClient {
    private final String SET_ACTION_URI = "/ispw/%s/sets/%s/%s"
	private final String GET_SET_URI = "/ispw/%s/sets/%s"
    private final String FALLBACK_SET_URI = "/ispw/%s/sets/%s/fallback"
	
	public static final String SET_STATE_DISPATCHED = "Dispatched"
	public static final String SET_STATE_EXECUTING = "Executing"
	public static final String SET_STATE_COMPLETE = "Complete"
	public static final String SET_STATE_CLOSED = "Closed"
	public static final String SET_STATE_FAILED = "Failed"
	public static final String SET_STATE_HELD = "Held"
	public static final String SET_STATE_RELEASED = "Released"
	public static final String SET_STATE_TERMINATED = "Terminated"
	public static final String SET_STATE_WAITING_APPROVAL = "Waiting-Approval"
	public static final String SET_STATE_WAITING_LOCK = "Waiting-Lock"

	public static final int POLLING_COUNT = 120
	public static final int POLLING_INTERVAL = 2000
	
	private Boolean skipWaitingForSet

    public SetClient(Properties props) {
        super(props)    // Configure CES connection and set Properties.
		skipWaitingForSet = Boolean.valueOf(props['skipWaitingForSet'])
    }

	/* Get set status on action performed on an ISPW set. */
	public String getAction(String setId, String action) {
		
		String uri = String.format(GET_SET_URI, serverId, setId)
		HttpResponse response = restHelper.doGetRequest(uri)
		Map<String, String> responseMap = restHelper.parseResponse(response)
		
		return responseMap.get("state")
	}
	
	//Set up polling
	private void performPolling(String setId, String action) {
		
		String getState = getAction(setId, action)
		
		int i = 0;
		for (; i < 60; i++) {
				
			Thread.sleep(POLLING_INTERVAL)
			
			getState = getAction(setId, action)

			if (getState.equals(SET_STATE_HELD) && action.equals("hold")) {
						println("Set " + setId + " successfully held")
						break
			
			}else if (getState.equals(SET_STATE_RELEASED) && action.equals("release")) {
						println("Set " + setId + " successfully released")
						break
			
			}else if (getState.equals(SET_STATE_WAITING_APPROVAL) && action.equals("unlock")) {
						println("Set " + setId + " successfully unlocked")
						break
			
			}else if (getState.equals(SET_STATE_WAITING_LOCK) && action.equals("lock")) {
						println("Set " + setId + " successfully locked")
						break
			
			}else if (getState.equals(SET_STATE_TERMINATED) && action.equals("delete")) {
						println("Set " + setId + " successfully deleted")
						break
			
			}

			if (i == 60) {
				logger.println("Warn - max timeout reached")
			}
		}
	}
		
	
	
    /* Perform an action on an ISPW set. */
    public Map<String, String> performAction(String setId, String action) {
        SetInfo setInfo = getSetInfo()  // Get SetInfo from the relevant Properties in CESClient.
        String uri = String.format(SET_ACTION_URI, serverId, setId, action)
        HttpResponse response = restHelper.doPostRequest(uri, setInfo.getJson())
        Map<String, String> responseMap = restHelper.parseResponse(response)
		
		if(!skipWaitingForSet) {
			
			println("Waiting for the completion of the set for this job...")
			performPolling(setId, action)
			
		}else {
			println("Skip waiting for the completion of the set for this job...")
		}
		
        return responseMap
    }

    /* Fallback an ISPW set. */
    public Map<String, String> fallback(String setId) {
        SetInfo setInfo = getSetInfo()  // Get SetInfo from the relevant Properties in CESClient.
        String uri = String.format(FALLBACK_SET_URI, serverId, setId)
        HttpResponse response = restHelper.doPostRequest(uri, setInfo.getJson())
        Map<String, String> responseMap = restHelper.parseResponse(response)
        return responseMap
    }

}
